#!/usr/local/bin/python3
# -*- coding: UTF-8 -*-
import sys
import os

def main(inputfile):
    if not os.path.exists(inputfile):
        print("***ERROR(py_merge_stationline.py): the inputfile is not exist.")
        return False
    outputfile = inputfile.strip() + "_tmp"

    # the list of baseline, its element is the list [A, B] where the A and B is the sitename of base and rove station.
    blList = []

    try:
        with open(inputfile, 'r') as f_input, open(outputfile, 'w', encoding='utf-8') as f_output:
            for line in f_input:
                # we need this line: RECORD1: GD25 GD22
                if len(line.strip()) <= 0:
                    continue
                if not "RECORD" in line:
                    continue
                value = line.split()
                if len(value) != 3:
                    continue
                sitname_1 = value[1].strip()
                sitname_2 = value[2].strip()

                if not ([sitname_1, sitname_2] in blList or [sitname_2, sitname_1] in blList):
                    blList.append([sitname_1, sitname_2])
            
            if len(blList) <= 0:
                print("***ERROR(py_merge_stationline.py): the inputfile contain no baseline RECORD.")
                return False
            
            baseline_num = len(blList)
            t_str = "nbaseline " + str(int(baseline_num)) + "\n"
            f_output.write(t_str)
            for i in range(baseline_num):
                t_str = "RECORD" + str(int(i + 1)) + ": " + blList[i][0] + " " + blList[i][1] + "\n"
                f_output.write(t_str)
    except:
        print("***ERROR(py_merge_stationline.py): There is a error in building or writing file.")
        return False
    return True

if __name__ == "__main__":
    message='命令错误'
    if len(sys.argv)<=1 :
        print(message)
        sys.exit()
    inputfile = sys.argv[1]
    if not main(inputfile):
        sys.exit()
