#!/usr/local/bin/python3

from http.client import FORBIDDEN
import matplotlib.pyplot as plt
import numpy as np
import sys
import math as mt

a = 6378137.0  
b = 6356752.31414 
a = 6378137.0 
f = 1 / 298.257223563 
b = 6356752.31414  
sqrt = mt.sqrt
sin = mt.sin
cos = mt.cos
atan = mt.atan
atan2 = mt.atan2
fabs = mt.fabs

def rad2angle(r):
    a = r * 180.0 / mt.pi
    return a

def angle2rad(a):
    r = a * mt.pi / 180.0
    return r

def XYZ2BLH(X, Y, Z):
    R = sqrt(X * X + Y * Y)
    B0 = atan2(Z, R)
    while 1:
        N = a / sqrt(1.0 - f * (2 - f) * sin(B0) * sin(B0))
        B_ = atan2(Z + N * f * (2 - f) * sin(B0), R)
        if (fabs(B_ - B0) < 1.0e-7):
            break
        B0 = B_
    L_ = atan2(Y, X)
    H = R / cos(B_) - N
    B = rad2angle(B_)
    L = rad2angle(L_)
    return B, L, H

def BLH2XYZ(B, L, H):
    B = angle2rad(B)  
    L = angle2rad(L)  
    e = sqrt((a ** 2 - b ** 2) / (a ** 2)) 
    N = a / sqrt(1 - e * e * sin(B) * sin(B)) 
    X = (N + H) * cos(B) * cos(L)
    Y = (N + H) * cos(B) * sin(L)
    Z = (N * (1 - e * e) + H) * sin(B)
    return X, Y, Z 


def readstation(filepath):
    Station=[]
    with open(filepath,'r',encoding='utf-8') as readfile:
        for line in readfile:
            station=[]
            data_line=line.strip('\n').split() 
            station.append(data_line[3])#name
            station.append(float(data_line[1]));station.append(float(data_line[0]));station.append(float(data_line[2]))#BLH
            XYZ=BLH2XYZ(float(data_line[1]),float(data_line[0]),float(data_line[2]))
            station.append(XYZ[0]);station.append(XYZ[1]);station.append(XYZ[2])#XYZ
            Station.append(station)
    return Station
pass

def findminstation(Station,Grid):
    stationname=Grid[7]
    station=[]
    for i in range(len(Station)):
        if stationname==Station[i][0]:
            for j in range(len(Station[i])):
                station.append(Station[i][j])
            break
    return station

def calculatedis(minstation,grid):
    X1=float(minstation[4]);Y1=float(minstation[5]);Z1=float(minstation[6])
    X2=float(grid[4]);Y2=float(grid[5]);Z2=float(grid[6])
    dx=X2-X1;dy=Y2-Y1;dz=Z2-Z1
    return float(mt.sqrt(dx*dx+dy*dy+dz*dz)/1000) #km

def readgrid(filepath,Station):
    Grid=[]
    with open(filepath,'r',encoding='utf-8') as readfile:
        num_row=0
        for line in readfile:
            if num_row>0:
                grid=[]
                data_line=line.strip('\n').split() 
                grid.append(data_line[0])
                grid.append(float(data_line[1]));grid.append(float(data_line[2]));grid.append(float(data_line[3]))#BLH
                XYZ=BLH2XYZ(float(data_line[1]),float(data_line[2]),float(data_line[3]))
                grid.append(XYZ[0]);grid.append(XYZ[1]);grid.append(XYZ[2])#XYZ
                grid.append(data_line[16])#basestation_name
                minstation=findminstation(Station,grid)
                dis=calculatedis(minstation,grid)
                grid.append(float(dis))
                #print(grid)
                Grid.append(grid)
            num_row=num_row+1
    return Grid
pass

def findlinepoint(Pointname,Station):
    point=[]  #寻找点位信息
    for i in range(len(Station)):
        if Pointname==Station[i][0]:
            for j in range(len(Station[i])):
                point.append(Station[i][j])
            break          
    return point  #name BLH  XYZ
    

def readline(filepath,Station):
    Line=[]
    with open(filepath,'r',encoding='utf-8') as readfile:
        for line in readfile:
                line1=[]
                data_line=line.strip('\n').split()
                for i in range(int(len(data_line))-1):
                    point=findlinepoint(data_line[i],Station)
                    line1.append(point)
                dis=calculatedis(line1[0],line1[1])
                line1.append(dis)
                Line.append(line1)
    return Line  #point1 point2 distance
                 
     
    

def paint_net(Station,Grid,Line):
    grid_name=[];grid_B=[];grid_L=[];grid_dis=[];grid_sta=[]
    for i in range(len(Grid)):
        grid_name.append(Grid[i][0]);grid_L.append(Grid[i][2]);grid_B.append(Grid[i][1]);grid_dis.append(Grid[i][8]);grid_sta.append(Grid[i][7])
    station_name=[];station_B=[];station_L=[]
    for i in range(len(Station)):
        station_name.append(Station[i][0]);station_L.append(Station[i][2]);station_B.append(Station[i][1]) 
    #station_range
    max_l=0;max_b=0;min_l=999;min_b=999
    for i in range(len(station_L)):
        if max_l<station_L[i]:max_l=station_L[i]
        if min_l>station_L[i]:min_l=station_L[i]
    for i in range(len(station_B)):
        if max_b<station_B[i]:max_b=station_B[i]
        if min_b>station_B[i]:min_b=station_B[i]   
    #vrs_range
    Max_l=0;Max_b=0;Min_l=999;Min_b=999
    for i in range(len(grid_L)):
        if Max_l<grid_L[i]:Max_l=grid_L[i]
        if Min_l>grid_L[i]:Min_l=grid_L[i]
    for i in range(len(grid_B)):
        if Max_b<grid_B[i]:Max_b=grid_B[i]
        if Min_b>grid_B[i]:Min_b=grid_B[i]  
    plt.figure(dpi=600)
    plt.tick_params(labelsize=1)#刻度值大小设置
    plt.subplot(1,2,1)
    plt.scatter(grid_L, grid_B, c='r', s=1,marker='.', label='Vrs') #散点图:x y 颜色 大小 类型 图例
    plt.scatter(station_L,station_B, c='b', s=2,marker='.', label='Station') #散点图:x y 颜色 大小 类型 图例
    for i in range(len(station_L)): 
        plt.text(float(station_L[i]+0.00005), float(station_B[i]+0.00005), station_name[i], fontdict={'size': 2, 'color': 'b'})

    #station-station
    for i in range(len(Line)):
        x=[];y=[]
        y.append(Line[i][0][1]);y.append(Line[i][1][1])
        x.append(Line[i][0][2]);x.append(Line[i][1][2])
        #print(x,y)
        plt.plot(x,y,'b-',alpha=0.5, linewidth=0.5)
        ymid=(y[0]+y[1])/2
        xmid=(x[0]+x[1])/2
        line_dis=str('{:.1f}'.format(Line[i][2]))+"km"
        plt.text(float(xmid), float(ymid), line_dis, fontdict={'size': 2, 'color': 'b'})      
    
    plt.xlabel("Longitude", fontdict={'size': 10}) #x轴名称
    plt.ylabel("Latitude", fontdict={'size': 10}) #y轴名称
    plt.title("StationNet", fontdict={'size': 10})   #标题
    plt.legend(loc="upper center")
    plt.ylim(min_b-0.5,max_b+0.5)
    plt.xlim(min_l-0.5,max_l+0.5)
    
    #fig2
    plt.subplots_adjust(left=None, bottom=None, right=None, top=None,wspace=0.3, hspace=None)#wspace:水平方向距离  #hspace：垂直方向
    plt.subplot(1,2,2)
    plt.tick_params(labelsize=10)
    plt.scatter(grid_L, grid_B, c='r', s=1,marker='.', label='Vrs') #散点图:x y 颜色 大小 类型 图例
    
    for i in range(len(grid_L)): 
        grid_form=grid_name[i]+":"+str('{:.1f}'.format(grid_dis[i]))+"km"+"-"+grid_sta[i]
        plt.text(float(grid_L[i]-0.005), float(grid_B[i]+0.00005), grid_form, fontdict={'size': 2, 'color': 'g'})
    
    plt.xlabel("Longitude", fontdict={'size': 10}) #x轴名称
    #plt.ylabel("B", fontdict={'size': 5}) #y轴名称
    plt.title("VrsForm", fontdict={'size': 10})   #标题
    plt.legend(loc="upper center")
    plt.ylim(Min_b-0.02,Max_b+0.02)
    plt.xlim(Min_l-0.02,Max_l+0.02)
    
    plt.savefig('station_net.png')
    plt.show() 
pass    
               
#python3 paint_point.py ./Stablh ./usebasestation.txt ./line_ite
if __name__  == '__main__':   
    message='命令错误'
    if len(sys.argv)<=1 :
        print(message)
        sys.exit()
    Station=readstation(sys.argv[1]) #name BLH XYZ
    print(Station)  
    Grid=readgrid(sys.argv[2],Station) #name BLH XYZ min_dis_sta_name distance
    #print(Grid)
    Line=readline(sys.argv[3],Station)
    #print(Line)
    paint_net(Station,Grid,Line)
    
    
    
