#!/bin/bash
# PROCESS CONFIGURE FOR RTK SMOOTH. CONFIGURE FORMAT
#### Configure File:
# TIME:  2021 7 16 0 0 0 86400
# OBSDIR:  /home/kpl/xps/
# BRDMDIR: /home/kpl/xps/brdm/
# NSIT: 6
# RECORD1: HB01 HB02
# RECORD2: HB02 HB03

#### XYZ File:
# HB01 x y z
# HB02 x y z
# HB03 x y z
main() {
    if [ $# -ne 3 ]; then
        printUsage
        exit 1
    fi
    if [ ! -f tiny.conf ];then
        echo "tiny.conf not exsit!"
        exit 1
    fi
    conf=$1
    xyzfile=$2
    bindir=$3

    # grep time
    iy=$(awk '{if($1~/^TIME/) print $2}' $conf)
    im=$(awk '{if($1~/^TIME/) print $3}' $conf)
    id=$(awk '{if($1~/^TIME/) print $4}' $conf)
    ih=$(awk '{if($1~/^TIME/) print $5}' $conf)
    ii=$(awk '{if($1~/^TIME/) print $6}' $conf)
    is=$(awk '{if($1~/^TIME/) print $7}' $conf)
    ss=$(awk '{if($1~/^TIME/) print $8}' $conf)
    sed -i 's/pos-time=.*/pos-time='${iy}' '${im}' '${id}' '${ih}' '${ii}' '${is}' '${ss}'/g' tiny.conf
    yy=${iy:2:2}
    smjd=$(ymd2mjd ${iy} ${im} ${id})
    sydoy=($(mjd2ydoy $smjd))
    echo "sody: ${sydoy[0]} ${sydoy[1]}"

    # get end day  #出现09会有bug
    isod=$(( 10#${ih}*3600+10#${im}*60+10#${is} ))
    dd=$(( (${isod}+${ss})/86401 ))
    emjd=$(( ${smjd}+${dd} ))
    eydoy=($(mjd2ydoy $emjd))
    echo "edoy: ${eydoy[0]} ${eydoy[1]}"

    # grep obsdir and brdmdir
    obsdir=$(awk '{if($1~/^obsdir/) print $2}' $conf)
    brdmdir=$(awk '{if($1~/^brdmdir/) print $2}' $conf)

    # grep nsit
    nbaseline=$(awk '{if($1~/^nbaseline/) print $2}' $conf)
    echo $nbaseline
    ## iterate each baseline
    for ((i=1;i<=${nbaseline};i++));do
        rove=$(awk '{if($1~/^RECORD'${i}':/) print $2}' $conf)
        base=$(awk '{if($1~/^RECORD'${i}':/) print $3}' $conf)
        echo $rove $base
        prepareConfigures $rove $base $xyzfile
        rove=$(echo $rove | tr 'A-Z' 'a-z')
        base=$(echo $base | tr 'A-Z' 'a-z')
        str=$(grep file-tempdir tiny.conf)  
        strarr=(${str//=/ })  
        tempdir=${strarr[1]}        
        outdir=${rove}_${base}
        infil="${tempdir}/amb_file_backward"

        ###ppp 
        # sed -i 's/pos-posmode =.*/pos-posmode = ppp-kinematic/g' tiny.conf
        # sed -i 's/pos-baseline=.*/pos-baseline='${base}' '${rove}'/g' tiny.conf
        # ./rtk "-k" "tiny.conf" "${obsdir}/2021/197/${rove}${ydoy[1]}0.${yy}o" "${obsdir}/whu%W%D.clk" "${brdmdir}/whu%W%D.sp3"

        ##########################################################################################################################################################
        # zgj/jyh smooth and iar here
        # TODO:need to specify ${infil} and ${outpath} in this shell beforehand, and meanwhile set the EWL/WL/N1 constrain file directory in tiny.conf.
        if [ -d ${tempdir} ];then
            echo "${tempdir} already exsit, so remove it"
            rm -r ${tempdir}
        fi
        mkdir ${tempdir}
        # rtk
        sed -i 's/pos-posmode=.*/pos-posmode=rtk-static/g' tiny.conf
        # ambfile 
        sed -i 's/file-ewlconfile=.*/file-ewlconfile='${tempdir}'\/EWL.con/g' tiny.conf
        sed -i 's/file-wlconfile=.*/file-wlconfile='${tempdir}'\/WL.con/g' tiny.conf
        sed -i 's/file-n1confile=.*/file-n1confile='${tempdir}'\/N1.con/g' tiny.conf

        flist=""
        for ((doy=${sydoy[1]};doy<=${eydoy[1]};doy++))
        do
            flist="${flist} ${obsdir}/${iy}/${doy}/${base}${doy}0.${yy}o ${obsdir}/${iy}/${doy}/${rove}${doy}0.${yy}o ${brdmdir}/brdm${doy}0.${yy}p"
        done
        echo "file_list: ${flist}"

        ./rtk -k tiny.conf -ti 15 ${flist}        
	     
        if [ ! -f ${infil} ];then
            echo "${infil} generage FAILED!"
            exit 1
        fi

        # backward_iar ewl
        ./backward_iar -f $infil -o $tempdir -m ewl
        # rtk ewl-cons
        ./rtk -k tiny.conf -ti 15 -con ewl ${flist} # 15 s interval

        # backward_iar wl
        ./backward_iar -f $infil -o $tempdir -m ewl
        ./backward_iar -f $infil -o $tempdir -m wl
        # rtk wl-cons
        ./rtk -k tiny.conf -ti 15 -con wl ${flist} # 15 s interval

        # backward_iar n1
        ./backward_iar -f $infil -o $tempdir -m ewl       
        ./backward_iar -f $infil -o $tempdir -m wl
        ./backward_iar -f $infil -o $tempdir -m n1
        # rtk n1-cons
        ./rtk -k tiny.conf -ti 5 -con n1 ${flist} # 5 s interval

        ##########################################################################################################################################################
        if [ -d ${outdir} ];then
            echo "${outdir} already exsit, so remove it"
            rm -r ${outdir}
        fi
        mkdir ${outdir}
        mv ${tempdir}/*.con ${outdir}
        mv ${tempdir}/res* ${outdir}
        mv ${tempdir}/amb* ${outdir}

        # extract atom
        if [ ! -d ../bbovrs/external_input ];then
            mkdir ../bbovrs/external_input -p
        fi
        if [ ! -f ${outdir}/res_file_backward ];then
            echo "***ERROR(rtk_smooth.sh): the residual file using to extrat atom is not exist: ${outdir}/res_file_backward"
            exit 1
        fi
        if [ ! -s ${outdir}/res_file_backward ];then
            echo "***ERROR(rtk_smooth.sh): the residual file using to extrat atom maybe a null file: ${outdir}/res_file_backward"
            exit 1
        fi

	    # extract atom
	    python3 ${bindir}/py_extractatom_form.py ${outdir}/res_file_backward ${outdir}/N1.con ../bbovrs/external_input

        echo "###MESSAGE(rtk_smooth.sh): Processing the NO.${i} finished"
  done
}
prepareConfigures() {
    # get data
    local rover=$1
    local base=$2
    local xyzfile=$3

    # read data
    # a=$(cat -n test.conf|grep pos-time=|awk '{print $1}')
    pos1=$(awk '{if($1~/^'${rover}'/) print $2}' $xyzfile)
    pos2=$(awk '{if($1~/^'${rover}'/) print $3}' $xyzfile)
    pos3=$(awk '{if($1~/^'${rover}'/) print $4}' $xyzfile)
    ref1=$(awk '{if($1~/^'${base}'/) print $2}' $xyzfile)
    ref2=$(awk '{if($1~/^'${base}'/) print $3}' $xyzfile)
    ref3=$(awk '{if($1~/^'${base}'/) print $4}' $xyzfile)

    # trans A to a
    rover=$(echo $rover | tr 'A-Z' 'a-z')
    base=$(echo $base | tr 'A-Z' 'a-z')

    # modify config file coord
    sed -i 's/pos-baseline=.*/pos-baseline='${rover}' '${base}'/g' tiny.conf    
    sed -i 's/ant-pos1=.*/ant-pos1='$pos1'/g' tiny.conf
    sed -i 's/ant-pos2=.*/ant-pos2='$pos2'/g' tiny.conf
    sed -i 's/ant-pos3=.*/ant-pos3='$pos3'/g' tiny.conf
    sed -i 's/ant-ref1=.*/ant-ref1='$ref1'/g' tiny.conf
    sed -i 's/ant-ref2=.*/ant-ref2='$ref2'/g' tiny.conf
    sed -i 's/ant-ref3=.*/ant-ref3='$ref3'/g' tiny.conf

    # create tempdir
    sed -i 's/file-tempdir=.*/file-tempdir='temp_${rover}_${base}'/g' tiny.conf
}
mjd2ydoy() {
    local mjd=$1
    local year=$((($mjd + 678940) / 365))
    local mjd0=$(ymd2mjd $year 1 1)
    local doy=$(($mjd - $mjd0))
    while [ $doy -le 0 ]; do
        year=$(($year - 1))
        mjd0=$(ymd2mjd $year 1 1)
        doy=$(($mjd - $mjd0 + 1))
    done
    printf "%d %03d\n" $year $doy
}
ymd2mjd() {
    local year=$1
    local mon=$((10#$2))
    local day=$((10#$3))
    [ $year -lt 100 ] && year=$((year + 2000))
    if [ $mon -le 2 ]; then
        mon=$(($mon + 12))
        year=$(($year - 1))
    fi
    local mjd=$(echo $year | awk '{print $1*365.25-$1*365.25%1-679006}')
    mjd=$(echo $mjd $year $mon $day | awk '{print $1+int(30.6001*($3+1))+2-int($2/100)+int($2/400)+$4}')
    #local mjd=$(bc <<< "$year*365.25 - $year*365.25 % 1 - 679006")
    #mjd=$(bc <<< "($mjd + (30.6001*($mon+1))/1 + 2 - $year/100 + $year/400 + $day)/1")
    echo $mjd
}
printUsage() {
    echo "-----------------------------------------------------"
    echo " GNSS POST-PROCESS RTK SCRIPT FOR TINYPPP PROGRAM"
    echo " Usage: rtk_smooth.sh conf xyzfile BINDIR SYS"
    echo "    ex: rtk_smooth.sh conf xyzfile bindir GRECJ"
    echo "-----------------------------------------------------"
}
main "$@"
