#!/bin/bash
#
# 
source /usr/dev/vdb1/zj/stu_version_write/zj_new/globalvar.sh

## the main function
main(){
    if [ "$#" -ne 1 ]; then
        printUsage
        exit 1
    fi
    homedir=$1
    echo ${bindir}
    # check workspace
    if [ ! -d ${homedir} ];then
        echo "***ERROR(sh_report.sh): the workspace of the whole process is not exist: ${homedir}"
        exit 1
    fi
    if [ ! -d ${homedir}/bbovrs ];then
        echo "***ERROR(sh_report.sh): the workspace of the bboVRS process is not exist: ${homedir}/bbovrs"
        exit 1
    fi
    if [ ! -d ${homedir}/bbovrs/external_input ];then
        echo "***ERROR(sh_report.sh): the workspace of the bboVRS process is not exist: ${homedir}/bbovrs/external_input"
        exit 1
    fi
    if [ ! -d ${homedir}/delaunary ];then
        echo "***ERROR(sh_report.sh): the workspace of the Delaunay process is not exist: ${homedir}/delaunary"
        exit 1
    fi
    if [ ! -d ${homedir}/rtk_smooth ];then
        echo "***ERROR(sh_report.sh): the workspace of the rtk_smooth process is not exist: ${homedir}/rtk_smooth"
        exit 1
    fi
    if [ ! -d ${homedir}/tmp ];then
        echo "***ERROR(sh_report.sh): the workspace of the tmp process is not exist: ${homedir}/tmp"
        exit 1
    fi

    if [ -d ${homedir}/report ];then
        echo "###MESSAGE(sh_report.sh): the folder of report is already exist, so remove it"
        rm -rf ${homedir}/report
    fi
    mkdir ${homedir}/report -p

    # MUST CHANGE WORKSPACE! the plot script can only output the graph to the current workspace.
    cd ${homedir}/report

    # check the file relased by data processing

    # build the report file
    outputfile="${homedir}/report/Report_DataProcess"
    if [ -f ${outputfile} ];then
        rm -f ${outputfile}
    fi

    # part 1: generate the text report of RTK and VRS
    python3 ${bindir}/py_generate_textreport.py ${homedir}
    if [ ! -s "${homedir}/report/Report_DataProcess" ];then
        echo "***ERROR(sh_report.sh): the text report of whole program is not generated."
    fi

    # part 2: generate the graphic report of RTK and VRS
    ## a) the graph of the atomsphere product extract by RTK
    for i in $( ls ${homedir}/bbovrs/external_input/ | grep amb );do
        if [ ! -s "${homedir}/bbovrs/external_input/${i}" ];then
            continue
        fi
        if [ -s "${homedir}/report/plot_atom.png" ];then
            rm -f "${homedir}/report/plot_atom.png"
        fi
        python3 ${bindir}/py_drawDDAtom.py 5 1 ${homedir}/bbovrs/external_input/${i}
        if [ ! -s "${homedir}/report/plot_atom.png" ];then
            echo "***ERROR(sh_report.sh): the graph of the baseline network and rove station is not generated."
        else
            mv ${homedir}/report/plot_atom.png ${homedir}/report/plot_extract_${i}.png
        fi
    done

    ## b) the graph of the atomsphere product interpolated by VRS
    for i in $( ls ${homedir}/bbovrs/ | grep _interp_ );do
        if [ ! -s "${homedir}/bbovrs/${i}" ];then
            continue
        fi
        if [ -s "${homedir}/report/plot_atom.png" ];then
            rm -f "${homedir}/report/plot_atom.png"
        fi
        python3 ${bindir}/py_drawDDAtom.py 30 2 ${homedir}/bbovrs/${i}
        if [ ! -s "${homedir}/report/plot_atom.png" ];then
            echo "***ERROR(sh_report.sh): the graph of the baseline network and rove station is not generated."
        else
            mv ${homedir}/report/plot_atom.png ${homedir}/report/plot_vrsinterp_${i}.png
        fi
    done

    ## c) the graph of the baseline network and rove station
    #python3 paint_point.py ./Stablh ./usebasestation.txt ./line_ite
    if [ -s ${homedir}/bbovrs/Stablh -a -s ${homedir}/tmp/newusebasestation.txt -a -s ${homedir}/bbovrs/line_ite ];then
        python3 ${bindir}/py_paint_net.py ${homedir}/bbovrs/Stablh ${homedir}/tmp/newusebasestation.txt ${homedir}/bbovrs/line_ite
        if [ ! -s "${homedir}/report/station_net.png" ];then
            echo "***ERROR(sh_report.sh): the graph of the baseline network and rove station is not generated."
        fi
    else
        echo "***ERROR(sh_report.sh): the programme maybe not finished, can not plot the graph of the baseline network and rove station."
    fi
}
function printUsage(){
    echo "RTK_POST script"
    echo "  Generate operation reports of below projects."
    echo "      1) RTK: generate the atmosphere products"
    echo "      2) VRS: generate the VRS observation"
    echo "Usage:"
    echo "  sh_process_report.sh workdir"
}
main "$@"
#main "/usr/dev/vdb1/zyq/source_code/rtk_post/work/work/cq2"
