#!/bin/bash
source /usr/dev/vdb1/zj/stu_version_write/zj_new/globalvar.sh
## the function of printing usage
function printUsage(){
    echo "RTK_POST script usage:"
    echo "  vrs_post.sh gnss.sit require.csv data.json label"
    echo "Example:"
    echo "  vrs_post.sh gnss.sit_out.txt CQ_input.csv data.json test"
    echo "Input:"
    echo "  gnss.sit    :   cooridinates for region network collection"
    echo "                                                            "
    echo "  cq.input    :   the require or setting of data processing, containing of the position of"
    echo "              VRS. The format is defined by BaiDu"
    echo "  label       :   the label of workspace, in order to avoid working directory conflicts"
    echo "  basenum     :   base station count that used to generate atomsphere"
    echo "  minduration :   minimum duration"
}
main(){
    if [ $# -ne 5 ]; then  
        echo "vrs_post.sh gnss.sit_out cq_input tag basenum minduration" 
        exit 1
    fi

    chmod +x ${bindir}/*
    chmod +x ${templatedir}/rtk_smooth/rtk
    chmod +x ${templatedir}/rtk_smooth/backward_iar

    local instation=${1}
    local inrequest=${2}
    local tag=${3}
    local maxbasenum=${4}
    local minduration=${5}

    # if [ -d ${workdir}/input ];then
    #     rm -r ${workdir}/input/
    # fi
    # if [ ! -d ${workdir}/input ];then
    #     mkdir ${workdir}/input -p
    # fi
    
    # if [ -d ${workdir}/${tag}${num_request} ];then
    #     rm -r ${workdir}/${tag}${num_request}
    # fi
    homedir=${workdir}/${tag}${num_request}
    mkdir ${homedir} -p 
    mkdir ${homedir}/bbovrs -p ## bbovrs directory 
    mkdir ${homedir}/delaunary -p ## delaunary directory 
    mkdir ${homedir}/rtk_smooth -p ## rtk_smooth directory 
    mkdir ${homedir}/obs -p ## obs directory 
    mkdir ${homedir}/tmp
    mkdir ${homedir}/tmp_test
    obsdir_p=${homedir}/obs
    cp -r ${templatedir}/bbovrs ${homedir}/

    ## prepare basic configuration file and software
    if [ -d ${templatedir}/bbovrs ];then
        cp -rf ${templatedir}/bbovrs ${homedir}
    else
        echo "***ERROR(vrs_post.sh): the configuration and software of bboVrs is not exist: ${templatedir}/bbovrs"
        exit 1
    fi
    if [ -d ${homedir}/bbovrs/external_input ];then
        rm -rf ${homedir}/bbovrs/external_input
        mkdir ${homedir}/bbovrs/external_input -p
    else
        mkdir ${homedir}/bbovrs/external_input -p
    fi
    if [ -d ${tablesdir} ];then
        cp -rf ${tablesdir} ${homedir}/bbovrs/
    else
        echo "***ERROR(vrs_post.sh): the table dirctory is not exist: ${tablesdir}"
        exit 1
    fi
    if [ -d ${templatedir}/rtk_smooth ];then
        cp -rf ${templatedir}/rtk_smooth ${homedir}
        chmod 755 ${homedir}/
    else
        echo "the configuration and software of rtk smooth is not exist: ${templatedir}/rtk_smooth"
        exit 1
    fi

    # (1) 读取文件
    echo "(1) VRS reading input and generating configuration ..."
    ## a) generate the station and request information from the unofficial format file.
    if [ ! -f "${homedir}/bbovrs/gnss.sit_out" ];then
        cp ${instation} ${homedir}/bbovrs/gnss.sit_out
    fi
    python3 ${bindir}/py_getinput_form.py ${instation} ${homedir}/tmp/outstation ${inrequest} ${homedir}/tmp/outrequest ${minduration}
    if [ ! -f ${homedir}/tmp/outstation ];then
        echo "***ERROR(vrs_post.sh): the output file of py_getinput_form.py is not exist: ${homedir}/tmp/outstation"
        exit 1
    fi
    if [ ! -f ${homedir}/tmp/outrequest ];then
        echo "***ERROR(vrs_post.sh): the output file of py_getinput_form.py is not exist: ${homedir}/tmp/outrequest"
        exit 1
    fi
    cp ${homedir}/tmp/outstation ${homedir}/tmp_test/outstation
    ## b) parse request and rewrite the data.json
    parseRequest ${homedir}/tmp/outrequest
    ## Tip: need to check whether rewrite data.json successfully or not
    #(2) Find the best triangle
    echo "(2) VRS Find the best triangle ..."
    for ((i=6;i<=${maxbasenum};i++));do
        #delete not have date station(updateoutstation)
        if [ -s ${homedir}/tmp_test/NODate.txt ];then
            updateoutstation ${homedir}/tmp_test/outstation ${homedir}/tmp_test/NODate.txt
        fi
        python3 ${bindir}/py_findmindis_station.py ${homedir}/tmp_test/outstation ${homedir}/tmp/outrequest ${homedir}/tmp_test/usebasestation.txt  ${i}
        if [ ! -f ${homedir}/tmp_test/usebasestation.txt ];then
            echo "***ERROR(vrs_post.sh): the out file of py_findmindis_station.py is not exist: ${homedir}/tmp_test/usebasestation.txt"
            exit 1
        fi

        python3 ${bindir}/py_getobsdw_form.py ${homedir}/tmp_test/usebasestation.txt ${homedir}/tmp_test/downstainform.txt  ${i}
        if [ ! -f ${homedir}/tmp_test/downstainform.txt ];then
            echo "***ERROR(vrs_post.sh): the out file of py_getobsdw_form.py is not exist: ${homedir}/tmp_test/downstainform.txt"
            exit 1
        fi
        
        python3 ${bindir}/py_getoutput_form.py ${homedir}/tmp_test/downstainform.txt ${homedir}/tmp_test/outstation ${obsdir_p} ${brdmdir} ${homedir}/tmp_test/conf.txt ${homedir}/tmp_test/xyzfile.txt ${homedir}/tmp_test/xyzblhfile
        mv ${homedir}/tmp_test/xyzblhfile ${homedir}/delaunary
        cd ${homedir}/delaunary
        ${bindir}/Delaunary
        cd ${homedir}
        mv ${homedir}/delaunary/outtriangles.txt ${homedir}/tmp_test/
        python3 ${bindir}/py_find_triangle.py ${homedir}/tmp/outrequest ${homedir}/tmp_test/xyzfile.txt ${homedir}/tmp_test/outtriangles.txt ${homedir}/tmp_test/stationline.txt ${homedir}/tmp_test/trianglestation.txt
        #add 三角网优化

        if [ -s ${homedir}/tmp_test/stationline.txt ];then
            echo "Find a Triangle for VRS grid!"
        fi
        if [ ! -s ${homedir}/tmp_test/stationline.txt ];then
            echo "Not Find a Triangle for VRS grid!"
            continue
        fi
        #update downstainform.txt
        updatedwstaform ${homedir}/tmp_test/downstainform.txt ${homedir}/tmp_test/trianglestation.txt
        #upgrade xyzfile.txt(trianglestation.txt)
        updatexyzfile ${homedir}/tmp_test/trianglestation.txt ${homedir}/tmp_test/xyzfile.txt
        #Judge whether the data exists
        ${bindir}/judge_obsexists.sh ${homedir}/tmp_test/downstainform.txt ${homedir}/tmp_test/outtype ${homedir}/tmp_test
        #add 坐标验证
        if [ -f ${homedir}/tmp_test/downstainform.txt ];then
            mv ${homedir}/tmp_test/*.txt ${homedir}/tmp/
            rm -r ${homedir}/tmp_test/
            echo "Successfully obtained triangle!"
            break
        fi
        if [ ! -f ${homedir}/tmp_test/downstainform.txt ];then
            mv ${homedir}/tmp_test/*
            echo "No suitable triangle found!"
        fi
    done
    #exit 1
    # (3) 下载数据
    echo "(3) VRS downloading observation ..."
    ${bindir}/down_allstation.sh ${homedir}/tmp/downstainform.txt ${homedir}/tmp/outtype.txt  ${homedir} #下载所有数据并且进行拼接
    
    # (4) 生成基线配置信息config.txt 
    echo "(4) VRS generating config.txt ..."
    # 生成config.txt
    cat tmp/conf.txt > tmp/config.txt
    cat tmp/stationline.txt >> tmp/config.txt

    # (5) rtk_smooth
    echo "(5) VRS${num_request} processing rtk_smooth ..."
    cd ${homedir}/rtk_smooth
    mkdir tmp
    sed -i "s#file-satantfile=.*#file-satantfile=${tablesdir}/igs14_2194.atx#g" tiny.conf
    sed -i "s#file-gptfile=.*#file-gptfile=${tablesdir}/gpt2_1wA.grd#g" tiny.conf
    cp ${homedir}/tmp/xyzfile.txt ${homedir}/rtk_smooth/xyz
    cp ${homedir}/tmp/config.txt ${homedir}/rtk_smooth/bs.config
    ExecuteWithoutOutput "${bindir}/rtk_smooth.sh bs.config xyz ${bindir}"
    #${bindir}/rtk_smooth.sh bs.config xyz ${bindir}

    # (6) 读取大气生成VRS
    echo "(6) VRS processing bbovrs ..."
    cd ${homedir}/bbovrs
    prepareConfigBBO
    python3 ${bindir}/py_getbaseline_form.py ${homedir}/tmp/stationline.txt ${homedir}/bbovrs/line_ite
    ${bindir}/bboVrs -mode vrs vrs.json -debug -post -obssmooth -vinput vrs_config.json

    # (7)生成报告
    echo "(7) VRS${num_request} Generate report ..."
    ${bindir}/sh_report.sh ${homedir}
    # (8) delete obs
    rm -r ${obsdir_p}/*
 
    echo "All Finish!"  
}
function prepareConfigBBO(){
    cat ${homedir}/rtk_smooth/xyz | awk '{print $1}' > sitlist
    ${bindir}/makeSes.py -f session.obj_out -l sitlist -o session.obj_out_tmp -m F -s GECJ --dztd 0.1 --qztd 0.0001 --pgps 10:0.02 --pgal 10:0.02 --pglo 10:0.02 --pcmp 10:0.02 --pqzs 10:0.02
    mv session.obj_out_tmp session.obj_out
    worktime=$(awk '{if($1~/^TIME/) printf("%d %d %d %d %d %d %d", $2,$3,$4,$5,$6,$7,$8)}' ${homedir}/tmp/config.txt)
    sed -i "s#\"time\":.*#\"time\":\"${worktime}\",#g" vrs.json
    sed -i "s#\"broad\":.*#\"broad\":\"${brdmdir}/brdm-DDD-0.-YY-p\",#g" vrs.json
    sed -i "s#\"obs\":.*#\"obs\":\"${obsdir_p}/-YYYY-/-DDD-/\",#g" vrs.json
    if [ ! -d ${homedir}/bbovrs/trace ];then
            mkdir ${homedir}/bbovrs/trace -p
    fi
    if [ ! -d ${homedir}/bbovrs/results ];then
            mkdir ${homedir}/bbovrs/results -p
    fi
    ## make up coordinates 
    ${bindir}/makeSit.py -l sitlist -o gnss.sit_out 
    ${bindir}/pos2sit.py --fxyz ${homedir}/rtk_smooth/xyz -s gnss.sit_out 
    mv gnss.sit_out_tmp gnss.sit_out
}
function updateoutstation(){
    local outstationdir=$1
    local nodatedir=$2
    station1=""
    if [ -f $nodatedir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            station1=`echo $sitn_low | awk -F " " '{print $1}'`
            #echo $station1
            num=0
            if [ -f $outstationdir ];then
                while read line1
                do
                    sitn_low1=$(echo $line1 | tr  '[A-Z]' '[a-z]') 
                    station=`echo $sitn_low1 | awk -F " " '{print $1}'`
                    let num=num+1
                    if [ "$station1" == "$station" ];then
                        sed -i "${num}d" $outstationdir
                        let num=num-1
                    fi
                done < $outstationdir
            fi

        done < $nodatedir
    fi

}
function updatexyzfile(){
    local trianglestationdir=$1
    local xyzfiledir=$2
    local num=0
    local station1_id=0
    local station2_id=0
    local station3_id=0
    if [ -f $trianglestationdir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            station1=`echo $sitn_low | awk -F " " '{print $1}'`
            #station1_id=$((10#${station1:2:2}))
            station2=`echo $sitn_low | awk -F " " '{print $2}'`
            #station2_id=$((10#${station2:2:2}))
            station3=`echo $sitn_low | awk -F " " '{print $3}'`
            #station3_id=$((10#${station3:2:2}))
        done < $trianglestationdir
    fi
    if [ -f $xyzfiledir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            let num=num+1
            stationname=`echo $sitn_low | awk -F " " '{print $1}'`
            #station_id=$((10#${stationname:2:2}))
            if [ "$stationname" == "$station1" ];then
                continue
            fi
            if [ "$stationname" == "$station2"  ];then
                continue
            fi
            if [ "$stationname" == "$station3"  ];then
                continue
            fi
            sed -i "${num}d" $xyzfiledir
            let num=num-1
        done < $xyzfiledir
    fi

}
function updatedwstaform(){
    local downstainformdir=$1   #测站下载数据信息地址
    local trianglestationdir=$2   #基线信息
    local num=0
    local station1_id=0
    local station2_id=0
    local station3_id=0
    if [ -f $trianglestationdir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            station1=`echo $sitn_low | awk -F " " '{print $1}'`
            #station1_id=$((10#${station1:2:2}))
            station2=`echo $sitn_low | awk -F " " '{print $2}'`
            #station2_id=$((10#${station2:2:2}))
            station3=`echo $sitn_low | awk -F " " '{print $3}'`
            #station3_id=$((10#${station3:2:2}))
        done < $trianglestationdir
    fi
    if [ -f $downstainformdir ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')
            let num=num+1
            stationname=`echo $sitn_low | awk -F " " '{print $13}'`
            #station_id=$((10#${stationname:2:2}))
            if [ "$stationname" == "$station1" ];then
                continue
            fi
            if [ "$stationname" == "$station2"  ];then
                continue
            fi
            if [ "$stationname" == "$station3"  ];then
                continue
            fi
            sed -i "${num}d" $downstainformdir
            let num=num-1
        done < $downstainformdir
    fi
}
function parseRequest(){
    local reqf=$1
    local indata=$2
    local num_row=0   #记录行数
    local doy=0
    #写控制文件
    if [ -f $reqf ];then
        while read line
        do
            if [ ${num_row} -gt 1 ];then
                #转换为格网点名字VR01....   坐标换成XYZ
                grid_lon=`echo $line | awk -F " " '{print $1}'`     #获取变量
                grid_lat=`echo $line | awk -F " " '{print $2}'`     
                grid_height=`echo $line | awk -F " " '{print $3}'`
                grid_id=`echo $line | awk -F " " '{print $16}'`      #直接是名字
                syear=`echo $line | awk -F " " '{print $4}'`
                smonth=`echo $line | awk -F " " '{print $5}'`
                sday=`echo $line | awk -F " " '{print $6}'`
                shour=`echo $line | awk -F " " '{print $7}'`
                sminute=`echo $line | awk -F " " '{print $8}'`
                ssec=`echo $line | awk -F " " '{print $9}'`

                mjd=$(ymd2mjd $syear $smonth $sday)
                YDOY=($(mjd2ydoy $mjd))   
                doy=${YDOY[1]} 
                if [ $doy -lt 100 ];then
                    doy="0$doy"
                fi

                eyear=`echo $line | awk -F " " '{print $10}'`
                emonth=`echo $line | awk -F " " '{print $11}'`
                eday=`echo $line | awk -F " " '{print $12}'`
                ehour=`echo $line | awk -F " " '{print $13}'`
                eminute=`echo $line | awk -F " " '{print $14}'`
                esec=`echo $line | awk -F " " '{print $15}'`
                alltime=`echo $line | awk -F " " '{print $17}'`
                tmp=`BLH2XYZ $grid_lat $grid_lon $grid_height`  #获取字符串
                arr_XYZ=($tmp)                                  #字符串转数组
                #对格网点进行更新
                ## should look into data.json
		        #ExecuteWithoutOutput "python3 ${bindir}/py_writereqs_form.py ${homedir}/bbovrs/data.json reqs=${grid_id}+${arr_XYZ[0]}+${arr_XYZ[1]}+${arr_XYZ[2]}+${syear}+${smonth}+${sday}+${shour}+${sminute}+${ssec}+$alltime"  #id XYZ starttime  alltime
                ExecuteWithoutOutput "python3 ${bindir}/py_writereqs_form.py ${homedir}/bbovrs/vrs_config.json reqs=${grid_id}+${arr_XYZ[0]}+${arr_XYZ[1]}+${arr_XYZ[2]}+${syear}+${smonth}+${sday}+${shour}+${sminute}+${ssec}+$alltime"
            fi
            let num_row=num_row+1
        done < $reqf
    fi
}
ExecuteWithoutOutput() {
        local cmd="$1"
        time=$(date +'%Y-%m-%d %H:%M:%S')
        $cmd >/dev/null 2>&1 
        if [ $? -eq 0 ]; then  
                #echo "脚本实现"
                return 0
        else
                echo "Execute ERROR!"
                return 1
        fi
}
BLH2XYZ(){
    local a=6378137.0  
    local b=6356752.31414 
    local pi=3.141592653589793
    local B=$1
    local L=$2
    local H=$3
    B=$(echo $B $pi|awk '{printf "%0.8f\n", $1*$2/180}')
    L=$(echo $L $pi|awk '{printf "%0.8f\n", $1*$2/180}')
    e=$(echo $a $b|awk '{printf "%0.8f\n",sqrt(($1^2-$2^2)/$1^2)}')
    N=$(echo $a $e $B|awk '{printf "%0.8f\n",$1/(sqrt(1-$2^2*sin($3)*sin($3)))}')
    X=$(echo $N $H $B $L|awk '{printf "%0.8f\n",($1+$2)*cos($3)*cos($4)}')
    Y=$(echo $N $H $B $L|awk '{printf "%0.8f\n",($1+$2)*cos($3)*sin($4)}')
    Z=$(echo $N $H $B $e|awk '{printf "%0.8f\n",($1*(1-$4*$4)+$2)*sin($3)}')
    echo $X $Y $Z
}
ymd2mjd() {
    local year=$1
    local mon=$((10#$2))
    local day=$((10#$3)) 
    [ $year -lt 100 ] && year=$((year + 2000))
    if [ $mon -le 2 ]; then
        mon=$(($mon + 12))
        year=$(($year - 1))
    fi
    local mjd=$(echo $year | awk '{print $1*365.25-$1*365.25%1-679006}')
    mjd=$(echo $mjd $year $mon $day | awk '{print $1+int(30.6001*($3+1))+2-int($2/100)+int($2/400)+$4}')
    echo $mjd
}
mjd2ydoy() {
    local mjd=$1
    local year=$((($mjd + 678940) / 365))
    local mjd0=$(ymd2mjd $year 1 1)
    local doy=$(($mjd - $mjd0))
    while [ $doy -le 0 ]; do
        year=$(($year - 1))
        mjd0=$(ymd2mjd $year 1 1)
        doy=$(($mjd - $mjd0 + 1))
    done
    printf "%d %03d\n" $year $doy
}
main "$@"   
